Index: code.cxx
===================================================================
--- code.cxx	(revision 6041)
+++ code.cxx	(working copy)
@@ -275,6 +275,34 @@
 int write_sourceview;
 extern Fl_Widget_Class_Type *current_widget_class;
 
+// recursively dump static code present exclusively in the current
+// declaration hierarchy
+static void write_static_code(Fl_Type* p) {
+  if (write_sourceview) p->header_line = (int)ftell(header_file);
+  p->write_static();
+  if (write_sourceview) {
+    p->header_line_end = (int)ftell(header_file);
+    if (p->header_line==p->header_line_end) p->header_line_end = -1;
+  }
+
+  for (Fl_Type* q = p->next; q && q->level > p->level; q = q->next) {
+    while (q->is_decl_block()) {
+      // skip nested declarations
+      Fl_Type* s;
+      for (s = q; s && s->level > q->level; s = s->next);
+      if (!s || s->level <= q->level) return;
+      q = s;
+    }
+
+    if (write_sourceview) q->header_line = (int)ftell(header_file);
+    q->write_static();
+    if (write_sourceview) {
+      q->header_line_end = (int)ftell(header_file);
+      if (q->header_line==q->header_line_end) q->header_line_end = -1;
+    }
+  }
+}
+
 // recursively dump code, putting children between the two parts
 // of the parent code:
 static Fl_Type* write_code(Fl_Type* p) {
@@ -283,6 +311,16 @@
     if (p->header_line_end==-1)
       p->header_line = (int)ftell(header_file);
   }
+
+  if(!p->parent || (p->parent->is_decl_block() && !p->parent->is_class())) {
+    // change the declaration environment for each block
+    included* old_root = included_root;
+    included_root = 0;
+    write_static_code(p);
+    delete included_root;
+    included_root = old_root;
+  }
+
   // write all code that come before the children code
   // (but don't write the last comment until the very end)
   if (!(p==Fl_Type::last && p->is_comment()))
@@ -411,22 +449,6 @@
     }
   }
   for (Fl_Type* p = first_type; p;) {
-    // write all static data for this & all children first
-    if (write_sourceview) p->header_line = (int)ftell(header_file);
-    p->write_static();
-    if (write_sourceview) {
-      p->header_line_end = (int)ftell(header_file);
-      if (p->header_line==p->header_line_end) p->header_line_end = -1;
-    }
-    for (Fl_Type* q = p->next; q && q->level > p->level; q = q->next) {
-      if (write_sourceview) q->header_line = (int)ftell(header_file);
-      q->write_static();
-      if (write_sourceview) {
-        q->header_line_end = (int)ftell(header_file);
-        if (q->header_line==q->header_line_end) q->header_line_end = -1;
-      }
-    }
-    // then write the nested code:
     p = write_code(p);
   }
 
